﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("ZDT4", "ZDT4 function as defined in http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/ [30.11.2015]")]
  [StorableClass]
  public class ZDT4 : ZDT {

    public override double[,] Bounds(int objectives) {
      double[,] bounds = new double[objectives, 2];
      bounds[0, 0] = 0; bounds[0, 1] = 1;
      for (int i = 1; i < objectives; i++) {
        bounds[i, 0] = -5;
        bounds[i, 1] = 5;
      }

      return bounds;
    }

    public override double BestKnownHypervolume(int objectives) {
      return 120 + 2.0 / 3;
    }

    [StorableConstructor]
    protected ZDT4(bool deserializing) : base(deserializing) { }
    protected ZDT4(ZDT4 original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ZDT4(this, cloner);
    }
    public ZDT4() : base() { }

    public override double[] Evaluate(RealVector r) {
      double g = 0;
      for (int i = 1; i < r.Length; i++) {
        double v = r[i];
        g += v * v - 10 * Math.Cos(4 * Math.PI * v);
      }
      g = 1.0 + 10.0 * (r.Length - 1) + g;
      double d = r[0] / g;
      return new double[] { r[0], 1-Math.Sqrt(d) };
    }
  }
}
